import requests
import time
import socket
from io import IOBase
from urllib.parse import urlsplit
from werkzeug.wrappers import Response
from requests_toolbelt import MultipartEncoder


class Handler:

    def __init__(self, status, body, json, headers, latency, target, abort):
        self._status = status
        self._body = body
        self._headers = headers
        if json is not None:
            self._body = json
            self._headers = self._headers or {}
            self._headers.update({'Content-Type': 'application/json'})
        self._latency = latency
        self._target = target
        self._abort = abort

    def status(self, default=200):
        return int(self._status) if (self._status is not None) else default

    def body(self, default=None):
        if self._body is None:
            return default
        elif isinstance(self._body, IOBase):
            self._body.seek(0)
            return self._body.read()
        else:
            return self._body

    def headers(self, default=None):
        headers = default if (default is not None) else {}
        if self._headers is not None:
            headers.update(self._headers)
        return headers

    def latency(self, default=0):
        return float(self._latency) if (self._latency is not None) else default

    def target(self):
        if (self._target is not None) and (self._target[-1] == '/'):
            return self._target[:-1]
        return self._target

    def abort(self):
        return bool(self._abort) if (self._abort is not None) else False

    def _prepare_request(self, request):
        url = self.target() + request.path
        if urlsplit(url).scheme == '':
            url = 'http://' + url

        params = dict(request.args.items())

        headers = dict(request.headers.items())
        if headers.get('Content-Length', None) == '':
            headers.pop('Content-Length', None)

        headers['Host'] = urlsplit(url).netloc

        data = dict(request.form.items())
        if request.headers.get('Content-Type', '').lower().startswith('multipart/form-data'):
            data = MultipartEncoder(data)
            headers['Content-Type'] = data.content_type

        return request.method, url, params, headers, data

    def _prepare_response(self, response):
        headers = dict(response.headers)
        # https://github.com/mitsuhiko/flask/issues/367
        headers.pop('Transfer-Encoding', None)
        return response.content, response.status_code, headers

    def __call__(self, request):
        if self.latency() != 0:
            time.sleep(self.latency())
        if self.abort() is True:
            raise socket.error
        if self.target() is None:
            return Response(self.body(), self.status(), self.headers())
        method, url, params, headers, data = self._prepare_request(request)
        response = requests.request(method, url, params=params, headers=headers, data=data)
        body, status, headers = self._prepare_response(response)
        return Response(self.body(body), self.status(status), self.headers(headers))



import re


class Rule:

    def __init__(self, method, route, params, headers):
        self._method = str(method).upper() if (method is not None) else 'ANY'
        self._route = str(route) if (route is not None) else '/'
        self._params = dict(params) if (params is not None) else {}
        self._headers = dict(headers) if (headers is not None) else {}

    def __match_method(self, method):
        return self._method == 'ANY' or method.upper() == self._method

    def __match_route(self, route):
        pattern = re.sub(r'\{(\w+)\}', '\w+', self._route, flags=re.IGNORECASE)
        return bool(re.compile(pattern).match(route))

    def __match_params(self, params):
        for key, value in self._params.items():
            if key not in params or value != params[key]:
                return False
        return True

    def __match_headers(self, headers):
        for key, value in self._headers.items():
            if key not in headers or value != headers[key]:
                return False
        return True

    def match(self, request):
        return self.__match_method(request.method) and \
               self.__match_route(request.path) and \
               self.__match_params(request.args) and \
               self.__match_headers(request.headers)



from setuptools import setup, find_packages

setup(
    name='jj',
    description='A simple testing tool for mocking HTTP responses',
    version='0.1.2',
    url='https://github.com/nikitanovosibirsk/jj',
    author='Nikita Tsvetkov',
    author_email='nikitanovosibirsk@yandex.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'Werkzeug==0.11.4',
        'requests==2.9.1',
        'requests-toolbelt==0.6.0'
    ]
)
